# $Id: emthreads.py,v 1.1 2011/05/04 07:34:03 irees Exp $
import Queue
import collections
import datetime
import json
import operator
import os
import time

from PyQt4 import QtCore

import emdash.config
import emdash.handlers



class ClockThread(QtCore.QThread):
	"""Emit a signal every second with timedelta from start time"""

	signal_tick = QtCore.pyqtSignal(str)

	def __init__(self, starttime=None, parent=None):
		QtCore.QThread.__init__(self, parent=parent)
		self.starttime = starttime
		
	def reset(self):
		self.starttime = time.time()
		
	def run(self):
		self.reset()
		
		while True:
			if self.starttime == None:
				t = ""
			else:
				t = int(time.time() - self.starttime)
				t = str(datetime.timedelta(seconds=t))

			self.signal_tick.emit(t)		
			time.sleep(1)


	@QtCore.pyqtSlot()	
	def longsleep(self):
		# print "longsleep"
		self.sleep(5)







class FSPollThread(QtCore.QThread):
	"""Watch a directory"""

	signal_newfile = QtCore.pyqtSignal(unicode, object)

	def __init__(self, path=None, parent=None): #queue=None, 
		"""Watches a directory for new files"""

		QtCore.QThread.__init__(self, parent=parent)
		self.path = path
		self.sleeptime = emdash.config.get('sleeptime')
		self.seen = set()


	@QtCore.pyqtSlot(str)
	def setpath(self, path):
		self.seen = set()
		self.path = unicode(path)
		self.start()


	def run(self):
		# print "FSPollThread: watching %s, sleeptime %s"%(self.path, self.sleeptime)
		while True:
			self.poll()
			time.sleep(self.sleeptime)


	def poll(self):
		"""get files, remove uploaded files from list, sort by creationtime"""

		if not self.path:
			return

		files = []
		# for f in os.listdir(self.path):
		for f in os.walk(self.path):
			for fi in f[2]:
				fi = os.path.join(f[0], fi)
				if fi not in self.seen:
					files.append(fi)

		files = [j[0] for j in sorted([(i,os.stat(i).st_ctime) for i in files], key=operator.itemgetter(1))]

		# ian: todo: There is no good way cross-platform to see if a file is still being written
		for f in files:
			self.signal_newfile.emit(f, None)
			self.seen.add(f)







class QualityThread(QtCore.QThread):
	"""Watch a directory"""

	signal_file_failure = QtCore.pyqtSignal(unicode, unicode)
	signal_file_status = QtCore.pyqtSignal(unicode, object)
	signal_file_success = QtCore.pyqtSignal(unicode, object)

	def __init__(self, db=None, filename=None, quality=None, name=None, parent=None):
		self.db = db

		self.quality = quality
		self.filename = filename
		self.name = name

		QtCore.QThread.__init__(self, parent=parent)


	def run(self):
		try:
			self.db.putrecordvalues(self.name, {"assess_image_quality":self.quality})
			self.signal_file_status.emit(self.filename, {"assess_image_quality":self.quality})

		except Exception, e:
			self.signal_exception.emit(unicode(e))






class UploadThread(QtCore.QThread):
	"""Upload a file"""

	signal_file_failure = QtCore.pyqtSignal(unicode, unicode)
	signal_file_status = QtCore.pyqtSignal(unicode, object)
	signal_file_success = QtCore.pyqtSignal(unicode, object)	

	def __init__(self, parent=None, queue=None, db=None):
		self.db = db
		self.queue = queue

		self.options = {
			"sidecar":True,
			"retry":False,
			"quiet":True
		}

		QtCore.QThread.__init__(self, parent=parent)



	def enqueue(self, filename, item):
		item["_status"] = "Queued"
		self.queue.put(item)
		self.signal_file_status.emit(filename, item)




	def report(self, msg=None, progress=None, f=None, pos=None, i=0):
		if progress == None:
			progress = 0.0

		if f:		
			self.signal_file_status.emit(f, {"_status":msg or progress, "_progress": progress})



	def run(self):		
		while True:
			try:
				self.upload()
			except Exception, e:
				print e
				break



	def upload(self):					

		item = self.queue.get()
		filename = item.get('_filename')
		name = item.get('_parent')


		# Note that _filename is used as either an actual filename, or an identifier
		# This causes the thread to terminate
		if filename == None:
			self.queue.task_done()
			raise Exception, "No more files"

		if name < 0:
			raise Exception, "UploadThread: Invalid Record name for upload: %s"%name

		# wait a small amount of time before proceeding..
		time.sleep(emdash.config.get('sleeptime'))

		# Remove the "meta-parameters" used for queue details
		rec = {}
		for k,v in item.items():
			if not k.startswith("_"):
				rec[k] = v												

		try:
			handler = emdash.handlers.gethandler(rectype=rec.get('rectype'), filename=filename)
			dbt = handler(db=self.db, filename=filename, name=name, options=self.options, applyparams=rec)
			dbt.addlistener(self.report)
			dbt.upload()

			# This is not very nice...
			# rec = ret.get(filename, dict()).get('name')
			rec = dbt.get_upload_target()
			if rec:
				rec = self.db.getrecord(rec)
				self.signal_file_success.emit(filename, rec)


		except Exception, e:
			print "UploadThread exception: %s"%e
			self.signal_file_failure.emit(filename, unicode(e))
			self.queue.put(item)
			time.sleep(5)


		self.queue.task_done()



__version__ = "$Revision: 1.1 $".split(":")[1][:-1].strip()




