# $Id: export.py,v 1.18 2011/04/23 21:20:07 irees Exp $
import time
import sys
import os
import operator

import xml.etree.ElementTree
etree = xml.etree.ElementTree

import xml.dom.minidom
md = xml.dom.minidom

# Standard View imports
import emen2.db.config
g = emen2.db.config.g()
from emen2.web.view import View
###


nodemap = {
	"root": ["emdEntry"],

	"emdEntry": ["deposition", "map", "supplement", "sample", "experiment", "processing", ],

	"deposition": ["mapReleaseDate", "fittedPDBEntryIDs", "primaryReference"],
		"primaryReference": ["journalArticle"],
			"journalArticle": ["authors", "articleTitle", "journal", "volume", "firstPage", "lastPage", "year", "externalReference"],

	"map": ["file", "dataType", "dimensions", "origin", "limit", "cell", "axisOrder", "statistics", "spaceGroupNumber", "details", "pixelSpacing", "contourLevel", "annotationDetails"],
		"dimensions": ["numColumns", "numRows", "numSections"],
		"origin": ["originCol", "originRow", "originSec"],
		"limit": ["limitCol", "limitRow", "limitSec"],
		"cell": ["cellA", "cellB", "cellC", "cellAlpha", "cellBeta", "cellGamma"],
		"axisOrder": ["axisOrderFast", "axisOrderMedium", "axisOrderSlow"],
		"statistics": ["minimum", "maximum", "average", "std"],
		"pixelSpacing": ["pixelX", "pixelY", "pixelZ"],

	"supplement": ["maskSet", "sliceSet", "figureSet", "fscSet"],

	"sample": ["numComponents", "aggregationState", "name", "compDegree", "molWtTheo", "sampleComponentList"],
		"sampleComponentList": ["sampleComponent"],
			"sampleComponent": ["protein"],
				"protein": ["refInterpro", "mutantFlag", "oligomericDetails", "refGo", "numCopies", "natSource", "engSource", "synName", "entry", "sciName", "molWtTheo"],
					"natSource": ["cell"],
					"engSource": ["vector", "expSystem"],


	"experiment": ["vitrification", "imaging", "imageScans", "fitting", "samplePreparation"],
		"vitrification": ["method", "cryogenName", "humidity", "instrument", "temperature"],
		"imaging": ["electronSource", "electronDose", "imagingMode", "nominalDefocusMin", "nominalDefocusMax", "illuminationMode", "temperatureMax", "specimenHolder", "details", "detector", "nominalCs", "temperature", "microscope", "date", "specimenHolderModel", "acceleratingVoltage", "nominalMagnification"],
		"imageScans": ["scanner", "numDigitalImages", "samplingSize"],
		"samplePreparation": ["buffer"],
			"buffer": ["details"],

	"processing": ["reconstruction", "emDataSet"],
		"reconstruction": ["algorithm", "software", "ctfCorrection", "resolutionByAuthor", "resolutionMethod"],
		"emDataSet": ["singleParticle"],
			"singleParticle": ["numProjections"]
}



def str_join(l):
	return ", ".join(map(str, l))




@View.register
class ExportEMDB(View):

	@View.add_matcher(r'^/export/emdb/(?P<name>\d+)/$')
	def init(self, name=None):
		self.template="/pages/export.emdb"
		self.title = "Export to EMDB"
		self.ctxt["name"]=int(name)

		self.root = int(name)
		exported = self.harvest()
		self.set_context_item("exported",exported)



	def harvest(self):

		# Basic Database Structure
		root = self.db.getrecord(self.root)
		subproject = self.db.getrecord(self.db.getparents(root.name, rectype=["subproject"])).pop()
		project = self.db.getrecord(self.db.getparents(root.name, rectype=["project"], recurse=2)).pop()


		####################
		# XML: emdEntry (root)
		e = etree.ElementTree(etree.Element("emdEntry", accessCode="ABC-123"))
		for i in nodemap["emdEntry"]:
			e2 = etree.Element(i)
			e.find("/").append(e2)


		####################
		# XML: emdEntry / deposition
		# <deposition>
		#  <status>REL</status>
		#  <depositionDate>2008-01-25</depositionDate>
		#  <headerReleaseDate>2008-02-12</headerReleaseDate>
		#  <mapReleaseDate>2009-04-14</mapReleaseDate>
		#  <title>Native, unliganded GroEL, D7 symmetrized, 4.2 A resolution 0.5 criterion</title>
		#  <authors>S.J.Ludtke,M.L.Baker,D.Chen,J.Song,D.T.Chuang,W.Chiu</authors>
		#  <fittedPDBEntryIdList>
		#    <fittedPDBEntryId>3cau</fittedPDBEntryId>
		#  </fittedPDBEntryIdList>
		#  <primaryReference published="true">
		#      <externalReference type="pubmed">18334219</externalReference>
		#    </journalArticle>
		#  </primaryReference>
		# </deposition>

		e_deposition = e.find('/deposition/')

		e_mapReleaseDate = etree.Element("mapReleaseDate")
		e_mapReleaseDate.text = time.strftime("%m-%d-%Y")
		e_deposition.append(e_mapReleaseDate)

		e_replaceExistingEntry = etree.Element("replaceExistingEntry")
		e_replaceExistingEntry.text = "false"
		e_deposition.append(e_replaceExistingEntry)


		# XML: emdEntry / deposition / primaryReference
		for rec in self.db.getrecord(self.db.getparents(root.name, rectype="publication*")):
			if rec.get('pmid'):
				e_primaryReference = etree.Element("primaryReference")
				e_journalarticle = etree.Element("journalArticle")
				e_externalReference = etree.Element("externalReference", type="pubmed")
				e_externalReference.text = str(rec.get('pmid'))
				e_journalarticle.append(e_externalReference)
				e_primaryReference.append(e_journalarticle)
				e_deposition.append(e_primaryReference)



		####################
		# XML: emdEntry / map
		# <map>
		# 	<file format="CCP4" sizeKb="32002" type="map">EMD-5001.map</file>
		# 	<contourLevel>0.597</contourLevel>
		# 	<annotationDetails>D7 structure of Groel at 4.2 Angstrom resolution structure</annotationDetails>
		# </map>

		e_map = e.find("/map/")

		mapbdo = self.db.getbinary(root.get("file_binary_image"))
		mapinfo = os.stat(mapbdo.get("filepath"))
		sizeKb = str(mapinfo.st_size / 1024)

		# mapemdata = EMAN2.EMData()
		# mapemdata.read_image(mapbdo.get("filepath"))
		# if mapemdata.get_ndim() != 3:
		# 	raise Exception, "Must be volume map"
		# mapheader = mapemdata.get_attr_dict()

		# XML: emdEntry / map / file
		e_file = etree.Element("file", format="MRC", sizeKb=sizeKb, type="map")
		e_map.append(e_file)



		####################
		# XML: emdEntry / supplement
		# <supplement>
		#   <maskSet/>
		#   <sliceSet/>
		#   <figureSet/>
		#   <fscSet/>
		# </supplement>
		e_supplement = e.find("/supplement/")


		# XML: emdEntry / supplement / masks


		e_maskSet = etree.Element("maskSet")
		maskset = self.db.getchildren(root.name, rectype="mask")

		for mask in maskset:
			maskbdos = self.db.getbinary([mask])
			for maskbdo in maskbdos:
				maskinfo = os.stat(maskbdo.get("filepath"))
				sizeKb = str(maskinfo.st_size / 1024)

				e_file = etree.Element("file", format="MRC", sizeKb=sizeKb, type="map")
				e_maskSet.append(e_file)

				e_details = etree.Element("details")
				e_details.text = self.db.renderview(mask, viewtype="recname")
				e_maskSet.append(e_details)


		e_sliceSet = etree.Element("sliceSet")
		e_figureSet = etree.Element("figureSet")
		e_fscSet = etree.Element("fscSet")

		e_supplement.append(e_maskSet)
		e_supplement.append(e_sliceSet)
		e_supplement.append(e_figureSet)
		e_supplement.append(e_fscSet)



		####################
		# XML: emdEntry / sample
		# <sample>
		#   <numComponents>1</numComponents>
		#   <aggregationState>singleParticle</aggregationState>
		#   <name>Native unliganded GroEL, residual ADP</name>
		#   <compDegree>Two back to back homo-heptameric rings</compDegree>
		#   <molWtTheo units="MDa">0.8</molWtTheo>
		#   <sampleComponentList>
		#     <sampleComponent componentID="1">
		#       <entry>protein</entry>
		#       <sciName>GroEL</sciName>
		#       <synName>GroEL</synName>
		#       <molWtTheo units="MDa">0.8</molWtTheo>
		#       <protein>
		#         <externalReferences>
		#           <refInterpro>IPR012723</refInterpro>
		#           <refGo>GO:0006457</refGo>
		#         </externalReferences>
		#         <mutantFlag>false</mutantFlag>
		#         <oligomericDetails>14-mer</oligomericDetails>
		#         <numCopies>14</numCopies>
		#         <natSource>
		#           <cell>E. coli</cell>
		#         </natSource>
		#         <engSource>
		#           <vector>pGroESL</vector>
		#           <expSystem>ESts  CG-712</expSystem>
		#         </engSource>
		#       </protein>
		#     </sampleComponent>
		#   </sampleComponentList>
		# </sample>


		e_sample = e.find("/sample/")

		molWtTheo = etree.Element("molWtTheo")

		molWtTheo.text = str(subproject.get("mass_specimen") / 1000)
		e_sample.append(molWtTheo)

		e_name = etree.Element("name")
		e_name.text = subproject.get("name_specimen")
		e_sample.append(e_name)

		e_compDegree = etree.Element("compDegree")
		e_compDegree.text = subproject.get("symmetry_particle")

		e_aggregationState = etree.Element("aggregationState")
		e_aggregationState.text = "singleParticle"
		e_sample.append(e_aggregationState)

		# Until I finalize the protocol..
		sample1 = {
			"entry": "protein",
			"sciName": "GroEL",
			"molWtTheo": 0.8,
			"refInterpro": ["IPR012723"],
			"refGo": ["GO:0006457"],
			"mutantFlag": False,
			"oligomericDetails": "14-mer",
			"numCopies": 14,
			"cell": "E. coli"
		}

		samplelist = [sample1]

		e_numComponents = etree.Element("numComponents")
		e_numComponents.text = str(len(samplelist))
		e_sample.append(e_numComponents)

		e_sampleComponentList = etree.Element("sampleComponentList")

		for count, sample in enumerate(samplelist):
			e_sampleComponent = etree.Element("sampleComponent", componentID="%s"%(count+1))
			e_protein = etree.Element("protein")

			if sample.get('refInterpro') or sample.get('refGo'):
				e_externalReferences = etree.Element('externalReferences')
				for i in sample.get('refInterpro', []):
					e_refInterpro = etree.Element("refInterpro")
					e_refInterpro.text = i
					e_externalReferences.append(e_refInterpro)

				for i in sample.get("refGo", []):
					e_refGo = etree.Element("refGo")
					e_refGo.text = i
					e_externalReferences.append(e_refGo)

				e_protein.append(e_externalReferences)


			if sample.get('mutantFlag'):
				e_mutantFlag = etree.Element("mutantFlag")
				e_mutantFlag.text = str(bool(sample.get('mutantFlag')))
				e_protein.append(e_mutantFlag)

			if sample.get('oligomericDetails'):
				e_oligomericDetails = etree.Element("oligomericDetails")
				e_oligomericDetails.text = sample.get('oligomericDetails')
				e_protein.append(e_oligomericDetails)

			if sample.get('numCopies'):
				e_numCopies = etree.Element("numCopies")
				e_numCopies.text = str(sample.get('numCopies'))
				e_protein.append(e_numCopies)

			if sample.get('cell'):
				e_natSource = etree.Element("natSource")
				e_cell = etree.Element("cell")
				e_cell.text = sample.get('cell')
				e_natSource.append(e_cell)
				e_protein.append(e_natSource)

			if sample.get('vector'):
				e_engSource = etree.Element("engSource")
				e_vector = etree.Element("vector")
				e_expSystem = etree.Element("expSystem")
				e_engSource.append(e_vector)
				e_engSource.append(e_expSystem)
				e_protein.append(e_engSource)

			if sample.get('synName'):
				e_synName = etree.Element("synName")
				e_synName.text = sample.get('synName')
				e_protein.append(e_synName)

			if sample.get('entry'):
				e_entry = etree.Element("entry")
				e_entry.text = sample.get('entry')
				e_protein.append(e_entry)

			if sample.get('sciName'):
				e_sciName = etree.Element("sciName")
				e_sciName.text = sample.get('sciName')
				e_protein.append(e_sciName)

			if sample.get('molWtTheo'):
				e_molWtTheo = etree.Element("molWtTheo")
				e_molWtTheo.text = str(sample.get('molWtTheo'))
				e_protein.append(e_molWtTheo)

			e_sampleComponent.append(e_protein)
			e_sample.append(e_sampleComponent)

		e_sample.append(e_sampleComponentList)



		####################
		# XML: emdEntry / experiment
		e_experiment = e.find("/experiment/")


		images = self.db.getchildren(root.name, rectype="image_capture*", recurse=-1)
		imagerecs = self.db.getrecord(images)


		# XML: emdEntry / experiment / vitrification
	# <vitrification>
	#   <method>Blot for 2 sec</method>
	#   <cryogenName>ETHANE</cryogenName>
	#   <humidity>100</humidity>
	#   <instrument>Vitrobot</instrument>
	#   <temperature units="Kelvin">100</temperature>
	# </vitrification>


		freezes = emen2.util.listops.combine(*self.db.getparents(images, rectype="freezing_vitrobot", recurse=2).values())
		freezerecs = self.db.getrecord(freezes)

		instrument = ", ".join(set(filter(None, [i.get('vitrification_device') for i in freezerecs])))
		blottimes = ", ".join(set(filter(None, [str(i.get("vitrobot_time_blot")) for i in freezerecs])))
		vitrobottemp = ", ".join(set(filter(None, [str(i.get("vitrobot_temp")+273) for i in freezerecs])))
		rh = ", ".join(set(filter(None, [str(i.get("vitrobot_humidity")) for i in freezerecs])))
		cryogen = ", ".join(set(filter(None, [str(i.get("cryogen")) for i in freezerecs])))

		e_vitrification = etree.Element("vitrification")

		e_method = etree.Element("method")
		e_method.text = "Blot for %s seconds"%blottimes
		e_vitrification.append(e_method)

		e_cryogenName = etree.Element("cryogenName")
		e_cryogenName.text = "Ethane" # cryogen
		e_vitrification.append(e_cryogenName)

		e_humidity = etree.Element("humidity")
		e_humidity.text = rh
		e_vitrification.append(e_humidity)

		e_instrument = etree.Element("instrument")
		e_instrument.text = instrument
		e_vitrification.append(e_instrument)

		e_temperature = etree.Element("temperature", units="Kelvin")
		e_temperature.text = vitrobottemp
		e_vitrification.append(e_temperature)

		e_experiment.append(e_vitrification)


		# XML: emdEntry / experiment / imaging
	# <imaging>
	#   <electronSource>FIELD EMISSION GUN</electronSource>
	#   <electronDose units="e/A**2">36</electronDose>
	#   <imagingMode>OTHER</imagingMode>
	#   <nominalDefocusMin units="nm">900</nominalDefocusMin>
	#   <temperatureMin units="Kelvin">4</temperatureMin>
	#   <nominalDefocusMax units="nm">2300</nominalDefocusMax>
	#   <illuminationMode>FLOOD BEAM</illuminationMode>
	#   <temperatureMax units="Kelvin">4</temperatureMax>
	#   <specimenHolder>Top entry</specimenHolder>
	#   <details>low dose on JEOL 3000SFF</details>
	#   <detector>Kodak SO163 film</detector>
	#   <nominalCs units="mm">1.6</nominalCs>
	#   <temperature units="Kelvin">4</temperature>
	#   <microscope>JEOL 3000SFF</microscope>
	#   <date>01-Jan-2005</date>
	#   <specimenHolderModel>OTHER</specimenHolderModel>
	#   <acceleratingVoltage units="kV">300</acceleratingVoltage>
	#   <nominalMagnification>60000</nominalMagnification>
	# </imaging>

		e_imaging = etree.Element("imaging")
		grid_imaging_recs = self.db.getrecord(emen2.util.listops.combine(*self.db.getparents(images, rectype="grid_imaging").values()))
		microscopyrecs = self.db.getrecord(emen2.util.listops.combine(*self.db.getparents(images, rectype="microscopy", recurse=-1).values()))
		microscoperecs = self.db.getrecord(emen2.util.listops.combine(*self.db.getparents(images, rectype="microscope", recurse=-1).values()))


		sn = set([None])
		mags = set([i.get("tem_magnification_set") for i in imagerecs]) - sn
		defocuses = set([i.get("ctf_defocus_set") for i in imagerecs]) - sn
		doses = set([i.get("tem_dose_rate") for i in imagerecs]) - sn
		temperatures = set([i.get("temperature_specimen") for i in grid_imaging_recs]) - sn
		detectors = set([i.get("ccd_id") for i in imagerecs]+[i.get("film_type") for i in imagerecs]) - sn
		cses = set([i.get("aberration_spherical") for i in microscoperecs]) - sn
		microscopenames = set([i.get("tem_name") for i in microscoperecs]) - sn
		dates = set([i.get("date_occurred") for i in imagerecs]) - sn
		voltages = set([i.get("lens_voltage_objective") for i in microscoperecs]) - sn



		e_electronDose = etree.Element("electronDose", units="e/A**2")
		e_electronDose.text = str(min(doses))
		e_imaging.append(e_electronDose)

		e_nominalDefocusMin = etree.Element("nominalDefocusMin", units="nM")
		e_nominalDefocusMin.text = str(min(defocuses)*1000)
		e_imaging.append(e_nominalDefocusMin)

		e_nominalDefocusMax = etree.Element("nominalDefocusMax", units="nM")
		e_nominalDefocusMax.text = str(max(defocuses)*1000)
		e_imaging.append(e_nominalDefocusMax)

		e_temperatureMin = etree.Element("temperatureMin", units="Kelvin")
		e_temperatureMin.text = str(min(temperatures))
		e_imaging.append(e_temperatureMin)

		e_temperatureMax = etree.Element("temperatureMax", units="Kelvin")
		e_temperatureMax.text = str(max(temperatures))
		e_imaging.append(e_temperatureMax)

		e_detector = etree.Element("detector")
		e_detector.text = str_join(detectors)
		e_imaging.append(e_detector)

		e_nominalCs = etree.Element("nominalCs")
		e_nominalCs.text = str(min(cses))
		e_imaging.append(e_nominalCs)

		e_temperature = etree.Element("temperature")
		e_temperature.text = str(max(temperatures))
		e_imaging.append(e_temperature)

		e_microscope = etree.Element("microscope")
		e_microscope.text = str_join(microscopenames)
		e_imaging.append(e_microscope)

		e_date = etree.Element("date")
		e_date.text = str_join(dates)
		e_imaging.append(e_date)

		e_acceleratingVoltage = etree.Element("acceleratingVoltage", units="kV")
		e_acceleratingVoltage.text = str_join(voltages)
		e_imaging.append(e_acceleratingVoltage)

		e_nominalMagnification = etree.Element("nominalMagnification")
		e_nominalMagnification.text = ", ".join(map(lambda x:str(x*1000), mags))
		e_imaging.append(e_nominalMagnification)

		e_details = etree.Element("details")
		e_details.text = "Low dose on %s"%e_microscope.text
		e_imaging.append(e_details)



		# MAG_FACTOR = 1.0
		# e_calibratedMagnification = etree.Element("calibratedMagnification")
		# e_calibratedMagnification.text = str()
		# e_imaging.append(e_calibratedMagnification)

		# Add these
		e_electronSource = etree.Element("electronSource")
		e_electronSource.text = "FIELD EMISSION GUN"
		e_imaging.append(e_electronSource)

		e_illuminationMode = etree.Element("illuminationMode")
		e_illuminationMode.text = "FLOOD BEAM"
		e_imaging.append(e_illuminationMode)

		e_specimenHolder = etree.Element("specimenHolder")
		e_specimenHolder.text = "Side entry"
		e_imaging.append(e_specimenHolder)

		e_specimenHolderModel = etree.Element("specimenHolderModel")
		e_specimenHolderModel.text = "OTHER"
		e_imaging.append(e_specimenHolderModel)

		e_imagingMode = etree.Element("imagingMode")
		e_imagingMode.text = "OTHER"
		e_imaging.append(e_imagingMode)


		e_experiment.append(e_imaging)


		# XML: emdEntry / experiment / imageScans
	# <imageScans>
	#   <scanner>NIKON SUPER COOLSCAN 9000</scanner>
	#   <numDigitalImages>135</numDigitalImages>
	#   <samplingSize units="microns">6.35</samplingSize>
	# </imageScans>

		scanner_film = set(filter(None, [i.get("scanner_film") for i in imagerecs])) - sn
		samplingSize = set(filter(None, [i.get("scan_step") for i in imagerecs])) - sn
		scans = set([i.name for i in imagerecs if i.rectype=="scan"])
		if scanner_film:
			e_imageScans = etree.Element("imageScans")
			e_scanner = etree.Element("scanner")
			e_scanner.text = str_join(scanner_film)
			e_numDigitalImages = etree.Element("numDigitalImages")
			e_numDigitalImages.text = str(len(scans))
			e_samplingSize = etree.Element("samplingSize")
			e_samplingSize.text = str_join(set(samplingSize)-sn)
			e_imageScans.append(e_scanner)
			e_imageScans.append(e_numDigitalImages)
			e_imageScans.append(e_samplingSize)
			e_experiment.append(e_imageScans)



		# XML: emdEntry / experiment / fitting
		#    <fitting/>
		e_fitting = etree.Element("fitting")
		e_experiment.append(e_fitting)



		# XML: emdEntry / experiment / samplePreparation
		# <samplePreparation>
		# 	      <buffer>
		# 	        <details>20 mM Tris.HCl, pH 7.5, 50 mM MgCl2</details>
		# 	      </buffer>
		# 	    </samplePreparation>
		e_samplePreparation = etree.Element("samplePreparation")

		samplebuffer = "20 mM Tris.HCl, pH 7.5, 50 mM MgCl2"
		if samplebuffer:
			e_buffer = etree.Element("buffer")
			e_buffer.text = samplebuffer
			e_samplePreparation.append(e_buffer)

		e_experiment.append(e_samplePreparation)




		####################
		# XML: emdEntry / processing
		e_processing = e.find("/processing/")


		# XML: emdEntry / processing / reconstruction
	# <reconstruction>
	#   <algorithm>projection matching</algorithm>
	#   <software>EMAN</software>
	#   <ctfCorrection>per micrograph</ctfCorrection>
	#   <resolutionByAuthor>4.2</resolutionByAuthor>
	#   <resolutionMethod>FSC at 0.5 cut-off</resolutionMethod>
	# </reconstruction>

		# Fix this
		r = {
			"algorithm": "projection matching",
			"software": "EMAN",
			"ctfCorrection": "per micrograph",
			"resolutionByAuthor": 4.2,
			"resolutionMethod": "FSC at 0.5 cut-off"
		}

		e_reconstruction = etree.Element("reconstruction")

		if r.get('algorithm'):
			e_algorithm = etree.Element("algorithm")
			e_algorithm.text = unicode(r.get('algorithm'))
			e_reconstruction.append(e_algorithm)

		if r.get('software'):
			e_software = etree.Element("software")
			e_software.text = unicode(r.get('software'))
			e_reconstruction.append(e_software)

		if r.get('ctfCorrection'):
			e_ctfCorrection = etree.Element("ctfCorrection")
			e_ctfCorrection.text = unicode(r.get('ctfCorrection'))
			e_reconstruction.append(e_ctfCorrection)

		if r.get('resolutionByAuthor'):
			e_resolutionByAuthor = etree.Element("resolutionByAuthor")
			e_resolutionByAuthor.text = unicode(r.get('resolutionByAuthor'))
			e_reconstruction.append(e_resolutionByAuthor)

		if r.get('resolutionMethod'):
			e_resolutionMethod = etree.Element("resolutionMethod")
			e_resolutionMethod.text = unicode(r.get('resolutionMethod'))
			e_reconstruction.append(e_resolutionMethod)

		e_processing.append(e_reconstruction)


		# XML: emdEntry / processing / emDataSet
	# <emDataSet>
	#   <singleParticle>
	#     <numProjections>20401</numProjections>
	#   </singleParticle>
	# </emDataSet>

		r = {
			'numProjections': 20401
		}

		e_emDataSet = etree.Element("emDataSet")

		if r.get('numProjections'):
			e_singleParticle = etree.Element('singleParticle')
			e_numProjections = etree.Element('numProjections')
			e_numProjections.text = unicode(r.get('numProjections'))
			e_singleParticle.append(e_numProjections)
			e_emDataSet.append(e_singleParticle)

		e_processing.append(e_emDataSet)
















		# XML: done.. print
		# pretty_print(etree.tostring(e.find("/")))

		pretty = etree.tostring(e.find("/"))
		pr2 = xml.dom.minidom.parseString(pretty)
		return pr2.toprettyxml()






















__version__ = "$Revision: 1.18 $".split(":")[1][:-1].strip()
